body {
    margin: 30px;
    max-width: 600px;
    position: relative;
    font-family: 'Open Sans', sans-serif;
}

.ancillary {
    padding: 15px;
    margin: 10px;
    background-color: lightgray;
}

.sidenote {
    position: absolute;
    left:650px;
    width: 200px;
}
