<html>
    <head>
        {% block head %}
        <title>{% block title %}{{ title }}{% endblock %}</title>
        <script type='text/javascript' src='https://s3.amazonaws.com/mturk-public/externalHIT_v1.js'></script>
        <link rel='stylesheet' href='bootstrap.css'>
        <link rel='stylesheet' href='main.css'>
        <link href='http://fonts.googleapis.com/css?family=Open+Sans:400italic,700italic,400,700' rel='stylesheet' type='text/css'>
        {% endblock head %}
    </head>
    <body>
        {% block question_title %}
        {% endblock %}

        {% block context_top %}
            {% if context_type.naive %}
                {% include 'context1_naive.html' %}
            {% endif %}
            {% if context_type.story %}
                {% include 'story_so_far.html' %}
            {% endif %}
        {% endblock %}

        <form name='mturk_form' id='mturk_form' action='{{ action_url }}' method='post'>
            <input type='hidden' value='ASSIGNMENT_ID_NOT_AVAILABLE' name='assignmentId' id='assignmentId' />
            {% block question_content %}
            {% endblock %}
            <div id='feedback'>
                <p>If you have any comments regarding this HIT, or found the instructions hard to understand, please let us know in the box below. We will use this information to improve the task in the future.</p>
                <textarea class='input-xxlarge' name='feedback' id='feedback' rows="5"></textarea>
            </div>
            <input class='btn btn-primary' type='submit' id='submitButton' />
        </form>

        <div class='ancillary'>
        {% block context_bottom %}
        {% endblock %}
        </div>

        {% block postjavascript %}
        <script src="http://ajax.googleapis.com/ajax/libs/jquery/1.9.0/jquery.min.js"></script>
        <script>window.jQuery || document.write('<script src="js/jquery-1.9.0.min.js"><\/script>')</script>
        <script language='Javascript'>
            turkSetAssignmentID();
        </script>
        {% endblock %}
    </body>
</html>

